/* test.c was originally generated by the autoSql program, which also 
 * generated test.h and test.sql.  This module links the database and
 * the RAM representation of objects. */

#include "common.h"
#include "linefile.h"
#include "dystring.h"
#include "jksql.h"
#include "test.h"

struct pt *ptCommaIn(char **pS, struct pt *ret)
/* Create a pt out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new pt */
{
char *s = *pS;
int i;

if (ret == NULL)
    AllocVar(ret);
ret->x = sqlSignedComma(&s);
ret->y = sqlSignedComma(&s);
*pS = s;
return ret;
}

void ptOutput(struct pt *el, FILE *f, char sep, char lastSep) 
/* Print out pt.  Separate fields with sep. Follow last field with lastSep. */
{
int i;
fprintf(f, "%d", el->x);
fputc(sep,f);
fprintf(f, "%d", el->y);
fputc(lastSep,f);
}

struct point *pointCommaIn(char **pS, struct point *ret)
/* Create a point out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new point */
{
char *s = *pS;
int i;

if (ret == NULL)
    AllocVar(ret);
sqlFixedStringComma(&s, ret->acc, sizeof(ret->acc));
ret->x = sqlSignedComma(&s);
ret->y = sqlSignedComma(&s);
ret->z = sqlSignedComma(&s);
s = sqlEatChar(s, '{');
ptCommaIn(&s, &ret->pt);
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
*pS = s;
return ret;
}

void pointFree(struct point **pEl)
/* Free a single dynamically allocated point such as created
 * with pointLoad(). */
{
struct point *el;

if ((el = *pEl) == NULL) return;
freez(pEl);
}

void pointFreeList(struct point **pList)
/* Free a list of dynamically allocated point's */
{
struct point *el, *next;

for (el = *pList; el != NULL; el = next)
    {
    next = el->next;
    pointFree(&el);
    }
*pList = NULL;
}

void pointOutput(struct point *el, FILE *f, char sep, char lastSep) 
/* Print out point.  Separate fields with sep. Follow last field with lastSep. */
{
int i;
if (sep == ',') fputc('"',f);
fprintf(f, "%s", el->acc);
if (sep == ',') fputc('"',f);
fputc(sep,f);
fprintf(f, "%d", el->x);
fputc(sep,f);
fprintf(f, "%d", el->y);
fputc(sep,f);
fprintf(f, "%d", el->z);
fputc(sep,f);
if (sep == ',') fputc('{',f);
ptCommaOut(&el->pt,f);
if (sep == ',') fputc('}',f);
fputc(lastSep,f);
}

struct polygon *polygonLoad(char **row)
/* Load a polygon from row fetched with select * from polygon
 * from database.  Dispose of this with polygonFree(). */
{
struct polygon *ret;
int sizeOne,i;
char *s;

AllocVar(ret);
ret->pointCount = sqlSigned(row[1]);
ret->id = sqlUnsigned(row[0]);
s = row[2];
for (i=0; i<ret->pointCount; ++i)
    {
    s = sqlEatChar(s, '{');
    slSafeAddHead(&ret->points, pointCommaIn(&s, NULL));
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
slReverse(&ret->points);
s = row[3];
AllocArray(ret->persp, ret->pointCount);
for (i=0; i<ret->pointCount; ++i)
    {
    s = sqlEatChar(s, '{');
    ptCommaIn(&s, &ret->persp[i]);
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
return ret;
}

struct polygon *polygonLoadAll(char *fileName) 
/* Load all polygon from a tab-separated file.
 * Dispose of this with polygonFreeList(). */
{
struct polygon *list = NULL, *el;
struct lineFile *lf = lineFileOpen(fileName, TRUE);
char *row[4];

while (lineFileRow(lf, row))
    {
    el = polygonLoad(row);
    slAddHead(&list, el);
    }
lineFileClose(&lf);
slReverse(&list);
return list;
}

struct polygon *polygonLoadByQuery(struct sqlConnection *conn, char *query)
/* Load all polygon from table that satisfy the query given.  
 * Where query is of the form 'select * from example where something=something'
 * or 'select example.* from example, anotherTable where example.something = 
 * anotherTable.something'.
 * Dispose of this with polygonFreeList(). */
{
struct polygon *list = NULL, *el;
struct sqlResult *sr;
char **row;

sr = sqlGetResult(conn, query);
while ((row = sqlNextRow(sr)) != NULL)
    {
    el = polygonLoad(row);
    slAddHead(&list, el);
    }
slReverse(&list);
sqlFreeResult(&sr);
return list;
}

void polygonSaveToDb(struct sqlConnection *conn, struct polygon *el, char *tableName, int updateSize)
/* Save polygon as a row to the table specified by tableName. 
 * As blob fields may be arbitrary size updateSize specifies the approx size
 * of a string that would contain the entire query. Arrays of native types are
 * converted to comma separated strings and loaded as such, User defined types are
 * inserted as NULL. Note that strings must be escaped to allow insertion into the database.
 * For example "autosql's features include" --> "autosql\'s features include" 
 * If worried about this use polygonSaveToDbEscaped() */
{
struct dyString *update = newDyString(updateSize);
dyStringPrintf(update, "insert into %s values ( %u,%d, NULL , NULL )", 
	tableName,  el->id,  el->pointCount);
sqlUpdate(conn, update->string);
freeDyString(&update);
}

void polygonSaveToDbEscaped(struct sqlConnection *conn, struct polygon *el, char *tableName, int updateSize)
/* Save polygon as a row to the table specified by tableName. 
 * As blob fields may be arbitrary size updateSize specifies the approx size.
 * of a string that would contain the entire query. Automatically 
 * escapes all simple strings (not arrays of string) but may be slower than polygonSaveToDb().
 * For example automatically copies and converts: 
 * "autosql's features include" --> "autosql\'s features include" 
 * before inserting into database. */ 
{
struct dyString *update = newDyString(updateSize);
dyStringPrintf(update, "insert into %s values ( %u,%d, NULL , NULL )", 
	tableName, el->id , el->pointCount );
sqlUpdate(conn, update->string);
freeDyString(&update);
}

struct polygon *polygonCommaIn(char **pS, struct polygon *ret)
/* Create a polygon out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new polygon */
{
char *s = *pS;
int i;

if (ret == NULL)
    AllocVar(ret);
ret->id = sqlUnsignedComma(&s);
ret->pointCount = sqlSignedComma(&s);
s = sqlEatChar(s, '{');
for (i=0; i<ret->pointCount; ++i)
    {
    s = sqlEatChar(s, '{');
    slSafeAddHead(&ret->points, pointCommaIn(&s,NULL));
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
slReverse(&ret->points);
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
s = sqlEatChar(s, '{');
AllocArray(ret->persp, ret->pointCount);
for (i=0; i<ret->pointCount; ++i)
    {
    s = sqlEatChar(s, '{');
    ptCommaIn(&s, &ret->persp[i]);
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
*pS = s;
return ret;
}

void polygonFree(struct polygon **pEl)
/* Free a single dynamically allocated polygon such as created
 * with polygonLoad(). */
{
struct polygon *el;

if ((el = *pEl) == NULL) return;
pointFreeList(&el->points);
freeMem(el->persp);
freez(pEl);
}

void polygonFreeList(struct polygon **pList)
/* Free a list of dynamically allocated polygon's */
{
struct polygon *el, *next;

for (el = *pList; el != NULL; el = next)
    {
    next = el->next;
    polygonFree(&el);
    }
*pList = NULL;
}

void polygonOutput(struct polygon *el, FILE *f, char sep, char lastSep) 
/* Print out polygon.  Separate fields with sep. Follow last field with lastSep. */
{
int i;
fprintf(f, "%u", el->id);
fputc(sep,f);
fprintf(f, "%d", el->pointCount);
fputc(sep,f);
/* Loading point list. */
    {
    struct point *it = el->points;
    if (sep == ',') fputc('{',f);
    for (i=0; i<el->pointCount; ++i)
        {
        fputc('{',f);
        pointCommaOut(it,f);
        it = it->next;
        fputc('}',f);
        fputc(',',f);
        }
    if (sep == ',') fputc('}',f);
    }
fputc(sep,f);
/* Loading pt list. */
    {
    struct pt *it = el->persp;
    if (sep == ',') fputc('{',f);
    for (i=0; i<el->pointCount; ++i)
        {
        fputc('{',f);
        ptCommaOut(&it[i],f);
        fputc('}',f);
        fputc(',',f);
        }
    if (sep == ',') fputc('}',f);
    }
fputc(lastSep,f);
}

struct polyhedron *polyhedronLoad(char **row)
/* Load a polyhedron from row fetched with select * from polyhedron
 * from database.  Dispose of this with polyhedronFree(). */
{
struct polyhedron *ret;
int sizeOne,i;
char *s;

AllocVar(ret);
ret->polygonCount = sqlSigned(row[2]);
ret->id = sqlUnsigned(row[0]);
s = cloneString(row[1]);
sqlStringArray(s, ret->names, 2);
s = row[3];
for (i=0; i<ret->polygonCount; ++i)
    {
    s = sqlEatChar(s, '{');
    slSafeAddHead(&ret->polygons, polygonCommaIn(&s, NULL));
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
slReverse(&ret->polygons);
s = row[4];
for (i=0; i<2; ++i)
    {
    s = sqlEatChar(s, '{');
    ptCommaIn(&s, &ret->screenBox[i]);
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
return ret;
}

struct polyhedron *polyhedronLoadAll(char *fileName) 
/* Load all polyhedron from a tab-separated file.
 * Dispose of this with polyhedronFreeList(). */
{
struct polyhedron *list = NULL, *el;
struct lineFile *lf = lineFileOpen(fileName, TRUE);
char *row[5];

while (lineFileRow(lf, row))
    {
    el = polyhedronLoad(row);
    slAddHead(&list, el);
    }
lineFileClose(&lf);
slReverse(&list);
return list;
}

struct polyhedron *polyhedronLoadByQuery(struct sqlConnection *conn, char *query)
/* Load all polyhedron from table that satisfy the query given.  
 * Where query is of the form 'select * from example where something=something'
 * or 'select example.* from example, anotherTable where example.something = 
 * anotherTable.something'.
 * Dispose of this with polyhedronFreeList(). */
{
struct polyhedron *list = NULL, *el;
struct sqlResult *sr;
char **row;

sr = sqlGetResult(conn, query);
while ((row = sqlNextRow(sr)) != NULL)
    {
    el = polyhedronLoad(row);
    slAddHead(&list, el);
    }
slReverse(&list);
sqlFreeResult(&sr);
return list;
}

void polyhedronSaveToDb(struct sqlConnection *conn, struct polyhedron *el, char *tableName, int updateSize)
/* Save polyhedron as a row to the table specified by tableName. 
 * As blob fields may be arbitrary size updateSize specifies the approx size
 * of a string that would contain the entire query. Arrays of native types are
 * converted to comma separated strings and loaded as such, User defined types are
 * inserted as NULL. Note that strings must be escaped to allow insertion into the database.
 * For example "autosql's features include" --> "autosql\'s features include" 
 * If worried about this use polyhedronSaveToDbEscaped() */
{
struct dyString *update = newDyString(updateSize);
char  *namesArray;
namesArray = sqlStringArrayToString(el->names, 2);
dyStringPrintf(update, "insert into %s values ( %u,'%s',%d, NULL , NULL )", 
	tableName,  el->id,  namesArray ,  el->polygonCount);
sqlUpdate(conn, update->string);
freeDyString(&update);
freez(&namesArray);
}

void polyhedronSaveToDbEscaped(struct sqlConnection *conn, struct polyhedron *el, char *tableName, int updateSize)
/* Save polyhedron as a row to the table specified by tableName. 
 * As blob fields may be arbitrary size updateSize specifies the approx size.
 * of a string that would contain the entire query. Automatically 
 * escapes all simple strings (not arrays of string) but may be slower than polyhedronSaveToDb().
 * For example automatically copies and converts: 
 * "autosql's features include" --> "autosql\'s features include" 
 * before inserting into database. */ 
{
struct dyString *update = newDyString(updateSize);
char  *namesArray;

namesArray = sqlStringArrayToString(el->names, 2);
dyStringPrintf(update, "insert into %s values ( %u,'%s',%d, NULL , NULL )", 
	tableName, el->id ,  namesArray , el->polygonCount );
sqlUpdate(conn, update->string);
freeDyString(&update);
}

struct polyhedron *polyhedronCommaIn(char **pS, struct polyhedron *ret)
/* Create a polyhedron out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new polyhedron */
{
char *s = *pS;
int i;

if (ret == NULL)
    AllocVar(ret);
ret->id = sqlUnsignedComma(&s);
s = sqlEatChar(s, '{');
for (i=0; i<2; ++i)
    {
    ret->names[i] = sqlStringComma(&s);
    }
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
ret->polygonCount = sqlSignedComma(&s);
s = sqlEatChar(s, '{');
for (i=0; i<ret->polygonCount; ++i)
    {
    s = sqlEatChar(s, '{');
    slSafeAddHead(&ret->polygons, polygonCommaIn(&s,NULL));
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
slReverse(&ret->polygons);
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
s = sqlEatChar(s, '{');
for (i=0; i<2; ++i)
    {
    s = sqlEatChar(s, '{');
    ptCommaIn(&s, &ret->screenBox[i]);
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
*pS = s;
return ret;
}

void polyhedronFree(struct polyhedron **pEl)
/* Free a single dynamically allocated polyhedron such as created
 * with polyhedronLoad(). */
{
struct polyhedron *el;

if ((el = *pEl) == NULL) return;
/* All strings in names are allocated at once, so only need to free first. */
if (el->names != NULL)
    freeMem(el->names[0]);
polygonFreeList(&el->polygons);
freez(pEl);
}

void polyhedronFreeList(struct polyhedron **pList)
/* Free a list of dynamically allocated polyhedron's */
{
struct polyhedron *el, *next;

for (el = *pList; el != NULL; el = next)
    {
    next = el->next;
    polyhedronFree(&el);
    }
*pList = NULL;
}

void polyhedronOutput(struct polyhedron *el, FILE *f, char sep, char lastSep) 
/* Print out polyhedron.  Separate fields with sep. Follow last field with lastSep. */
{
int i;
fprintf(f, "%u", el->id);
fputc(sep,f);
if (sep == ',') fputc('{',f);
for (i=0; i<2; ++i)
    {
    if (sep == ',') fputc('"',f);
    fprintf(f, "%s", el->names[i]);
    if (sep == ',') fputc('"',f);
    fputc(',', f);
    }
if (sep == ',') fputc('}',f);
fputc(sep,f);
fprintf(f, "%d", el->polygonCount);
fputc(sep,f);
/* Loading polygon list. */
    {
    struct polygon *it = el->polygons;
    if (sep == ',') fputc('{',f);
    for (i=0; i<el->polygonCount; ++i)
        {
        fputc('{',f);
        polygonCommaOut(it,f);
        it = it->next;
        fputc('}',f);
        fputc(',',f);
        }
    if (sep == ',') fputc('}',f);
    }
fputc(sep,f);
/* Loading pt list. */
    {
    struct pt *it = el->screenBox;
    if (sep == ',') fputc('{',f);
    for (i=0; i<2; ++i)
        {
        fputc('{',f);
        ptCommaOut(&it[i],f);
        fputc('}',f);
        fputc(',',f);
        }
    if (sep == ',') fputc('}',f);
    }
fputc(lastSep,f);
}

struct twoPoint *twoPointCommaIn(char **pS, struct twoPoint *ret)
/* Create a twoPoint out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new twoPoint */
{
char *s = *pS;
int i;

if (ret == NULL)
    AllocVar(ret);
sqlFixedStringComma(&s, ret->name, sizeof(ret->name));
s = sqlEatChar(s, '{');
ptCommaIn(&s, &ret->a);
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
s = sqlEatChar(s, '{');
ptCommaIn(&s, &ret->b);
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
s = sqlEatChar(s, '{');
for (i=0; i<2; ++i)
    {
    s = sqlEatChar(s, '{');
    ptCommaIn(&s, &ret->points[i]);
    s = sqlEatChar(s, '}');
    s = sqlEatChar(s, ',');
    }
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
*pS = s;
return ret;
}

void twoPointOutput(struct twoPoint *el, FILE *f, char sep, char lastSep) 
/* Print out twoPoint.  Separate fields with sep. Follow last field with lastSep. */
{
int i;
if (sep == ',') fputc('"',f);
fprintf(f, "%s", el->name);
if (sep == ',') fputc('"',f);
fputc(sep,f);
if (sep == ',') fputc('{',f);
ptCommaOut(&el->a,f);
if (sep == ',') fputc('}',f);
fputc(sep,f);
if (sep == ',') fputc('{',f);
ptCommaOut(&el->b,f);
if (sep == ',') fputc('}',f);
fputc(sep,f);
/* Loading pt list. */
    {
    struct pt *it = el->points;
    if (sep == ',') fputc('{',f);
    for (i=0; i<2; ++i)
        {
        fputc('{',f);
        ptCommaOut(&it[i],f);
        fputc('}',f);
        fputc(',',f);
        }
    if (sep == ',') fputc('}',f);
    }
fputc(lastSep,f);
}

struct stringArray *stringArrayLoad(char **row)
/* Load a stringArray from row fetched with select * from stringArray
 * from database.  Dispose of this with stringArrayFree(). */
{
struct stringArray *ret;
int sizeOne,i;
char *s;

AllocVar(ret);
ret->numNames = sqlSigned(row[0]);
sqlStringDynamicArray(row[1], &ret->names, &sizeOne);
assert(sizeOne == ret->numNames);
return ret;
}

struct stringArray *stringArrayLoadAll(char *fileName) 
/* Load all stringArray from a tab-separated file.
 * Dispose of this with stringArrayFreeList(). */
{
struct stringArray *list = NULL, *el;
struct lineFile *lf = lineFileOpen(fileName, TRUE);
char *row[2];

while (lineFileRow(lf, row))
    {
    el = stringArrayLoad(row);
    slAddHead(&list, el);
    }
lineFileClose(&lf);
slReverse(&list);
return list;
}

struct stringArray *stringArrayLoadByQuery(struct sqlConnection *conn, char *query)
/* Load all stringArray from table that satisfy the query given.  
 * Where query is of the form 'select * from example where something=something'
 * or 'select example.* from example, anotherTable where example.something = 
 * anotherTable.something'.
 * Dispose of this with stringArrayFreeList(). */
{
struct stringArray *list = NULL, *el;
struct sqlResult *sr;
char **row;

sr = sqlGetResult(conn, query);
while ((row = sqlNextRow(sr)) != NULL)
    {
    el = stringArrayLoad(row);
    slAddHead(&list, el);
    }
slReverse(&list);
sqlFreeResult(&sr);
return list;
}

void stringArraySaveToDb(struct sqlConnection *conn, struct stringArray *el, char *tableName, int updateSize)
/* Save stringArray as a row to the table specified by tableName. 
 * As blob fields may be arbitrary size updateSize specifies the approx size
 * of a string that would contain the entire query. Arrays of native types are
 * converted to comma separated strings and loaded as such, User defined types are
 * inserted as NULL. Note that strings must be escaped to allow insertion into the database.
 * For example "autosql's features include" --> "autosql\'s features include" 
 * If worried about this use stringArraySaveToDbEscaped() */
{
struct dyString *update = newDyString(updateSize);
char  *namesArray;
namesArray = sqlStringArrayToString(el->names, el->numNames);
dyStringPrintf(update, "insert into %s values ( %d,'%s')", 
	tableName,  el->numNames,  namesArray );
sqlUpdate(conn, update->string);
freeDyString(&update);
freez(&namesArray);
}

void stringArraySaveToDbEscaped(struct sqlConnection *conn, struct stringArray *el, char *tableName, int updateSize)
/* Save stringArray as a row to the table specified by tableName. 
 * As blob fields may be arbitrary size updateSize specifies the approx size.
 * of a string that would contain the entire query. Automatically 
 * escapes all simple strings (not arrays of string) but may be slower than stringArraySaveToDb().
 * For example automatically copies and converts: 
 * "autosql's features include" --> "autosql\'s features include" 
 * before inserting into database. */ 
{
struct dyString *update = newDyString(updateSize);
char  *namesArray;

namesArray = sqlStringArrayToString(el->names, el->numNames);
dyStringPrintf(update, "insert into %s values ( %d,'%s')", 
	tableName, el->numNames ,  namesArray );
sqlUpdate(conn, update->string);
freeDyString(&update);
}

struct stringArray *stringArrayCommaIn(char **pS, struct stringArray *ret)
/* Create a stringArray out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new stringArray */
{
char *s = *pS;
int i;

if (ret == NULL)
    AllocVar(ret);
ret->numNames = sqlSignedComma(&s);
s = sqlEatChar(s, '{');
AllocArray(ret->names, ret->numNames);
for (i=0; i<ret->numNames; ++i)
    {
    ret->names[i] = sqlStringComma(&s);
    }
s = sqlEatChar(s, '}');
s = sqlEatChar(s, ',');
*pS = s;
return ret;
}

void stringArrayFree(struct stringArray **pEl)
/* Free a single dynamically allocated stringArray such as created
 * with stringArrayLoad(). */
{
struct stringArray *el;

if ((el = *pEl) == NULL) return;
/* All strings in names are allocated at once, so only need to free first. */
if (el->names != NULL)
    freeMem(el->names[0]);
freeMem(el->names);
freez(pEl);
}

void stringArrayFreeList(struct stringArray **pList)
/* Free a list of dynamically allocated stringArray's */
{
struct stringArray *el, *next;

for (el = *pList; el != NULL; el = next)
    {
    next = el->next;
    stringArrayFree(&el);
    }
*pList = NULL;
}

void stringArrayOutput(struct stringArray *el, FILE *f, char sep, char lastSep) 
/* Print out stringArray.  Separate fields with sep. Follow last field with lastSep. */
{
int i;
fprintf(f, "%d", el->numNames);
fputc(sep,f);
if (sep == ',') fputc('{',f);
for (i=0; i<el->numNames; ++i)
    {
    if (sep == ',') fputc('"',f);
    fprintf(f, "%s", el->names[i]);
    if (sep == ',') fputc('"',f);
    fputc(',', f);
    }
if (sep == ',') fputc('}',f);
fputc(lastSep,f);
}

/* -------------------------------- End autoSql Generated Code -------------------------------- */

